#include "PreCompiledHeaders.hh"
#include "Lib/Document/CRTPSubDocument.hh"
#include "Lib/JsonBuilder.hh"

using namespace Vivy;

// Init a ASS sub-document from a file
void
AssSubDocument::initFromPath(const QString &path)
{
    this->ass_container = VVLib::ASSContainerFromFile(path.toUtf8().data());
    for (size_t i = 0; i < VVLib::ASSContainerGetLinesCount(this->ass_container); i += 1) {
        this->lines.push_back(VVLib::ASSContainerGetLineAt(this->ass_container, i));
    }
    for (size_t i = 0; i < VVLib::ASSContainerGetStylesCount(this->ass_container); i += 1) {
        this->styles.push_back(VVLib::ASSContainerGetStyleByName(
            this->ass_container, VVLib::ASSContainerGetStyleNameAt(this->ass_container, i)));
    }
}

AssSubDocument::~AssSubDocument() noexcept { VVLib::ASSContainerDrop(this->ass_container); }

QJsonDocument
AssSubDocument::getProperties() const noexcept
{
    QJsonObject styleObject;
    for (size_t i = 0; i < VVLib::ASSContainerGetStylesCount(this->ass_container); i += 1) {
        const auto view = VVLib::ASSContainerGetStyleNameAt(this->ass_container, i);
        styleObject.insert(QString::number(i),
                           QString::fromUtf8(view.str, static_cast<qsizetype>(view.len)));
    }

    return JsonBuilder::createOrderedJsonDocument({
        { "File", filePath },
        { "Styles", styleObject },
    });
}

void
AssSubDocument::setFilePath(QString filepath) noexcept
{
    filePath = filepath;
}
