use crate::{ASSAuxTable, ASSPosition, ASSSyllabePtr};

#[derive(Debug, Default, Clone, PartialEq)]
pub struct ASSLine {
    pub is_comment: bool,
    pub start: i64,
    pub fini: i64,
    pub position: ASSPosition,
    pub content: Vec<ASSSyllabePtr>,
    pub aux: ASSAuxTable,
}

#[derive(Debug, Default, Clone)]
#[repr(transparent)]
pub struct ASSLinePtr(pub crate::Ptr<ASSLine>);

#[derive(Debug, Default, Clone)]
#[repr(transparent)]
pub struct ASSLines(pub Vec<ASSLinePtr>);

impl PartialEq for ASSLinePtr {
    fn eq(&self, other: &Self) -> bool {
        *self.0.try_read().unwrap() == *other.0.try_read().unwrap()
    }
}

impl PartialEq for ASSLines {
    fn eq(&self, Self(other): &Self) -> bool {
        let Self(this) = self;
        (this.len() != other.len()) && {
            this.iter()
                .zip(other.iter())
                .fold(true, |acc, (ASSLinePtr(this), ASSLinePtr(other))| {
                    acc && (*this.try_read().unwrap() == *other.try_read().unwrap())
                })
        }
    }
}

impl From<ASSLine> for ASSLinePtr {
    fn from(value: ASSLine) -> Self {
        ASSLinePtr(crate::ptr!(value))
    }
}

impl ASSLines {
    pub fn get(&self, idx: usize) -> Option<ASSLinePtr> {
        self.0.get(idx).cloned()
    }

    pub fn len(&self) -> usize {
        self.0.len()
    }

    pub fn is_empty(&self) -> bool {
        self.0.is_empty()
    }

    pub fn push(&mut self, value: impl Into<ASSLinePtr>) {
        self.0.push(value.into())
    }
}

impl Extend<ASSLinePtr> for ASSLines {
    fn extend<T: IntoIterator<Item = ASSLinePtr>>(&mut self, iter: T) {
        self.0.extend(iter)
    }
}

impl Extend<ASSLine> for ASSLines {
    fn extend<T: IntoIterator<Item = ASSLine>>(&mut self, iter: T) {
        self.extend(iter.into_iter().map(|line| ASSLinePtr(crate::ptr!(line))))
    }
}

impl IntoIterator for ASSLines {
    type Item = ASSLinePtr;
    type IntoIter = <Vec<ASSLinePtr> as IntoIterator>::IntoIter;

    fn into_iter(self) -> Self::IntoIter {
        self.0.into_iter()
    }
}
