mod line;
mod syllabe;

pub use self::{line::*, syllabe::*};

use crate::{definitions::ScriptInfoKey, ASSStyle};
use std::collections::HashMap;

#[derive(Debug, Clone)]
pub struct ASSContainer {
    pub lines: ASSLines,
    pub script_info: HashMap<ScriptInfoKey, String>,
    pub styles: HashMap<String, ASSStyle>,
}

#[derive(Debug, Clone)]
#[repr(transparent)]
pub struct ASSContainerPtr(pub crate::Ptr<ASSContainer>);

impl ASSContainer {
    /// Create an ASS container from its parts, they must be valide!
    pub(crate) fn from_parts(
        lines: impl IntoIterator<Item = ASSLine>,
        script_info: HashMap<ScriptInfoKey, String>,
        styles: HashMap<String, ASSStyle>,
    ) -> Self {
        Self {
            lines: ASSLines(lines.into_iter().map(|line| ASSLinePtr(crate::ptr!(line))).collect()),
            script_info,
            styles,
        }
    }
}

impl From<ASSContainer> for ASSContainerPtr {
    fn from(value: ASSContainer) -> Self {
        Self(crate::ptr!(value))
    }
}
