use crate::{ASSAuxTable, ASSPosition};

#[derive(Debug, Default, Clone, PartialEq)]
pub struct ASSSyllabe {
    pub position: ASSPosition,
    pub content: String,
    pub aux: ASSAuxTable,
    pub start: i64,
    pub fini: i64,
}

#[derive(Debug, Default, Clone)]
#[repr(transparent)]
pub struct ASSSyllabePtr(pub crate::Ptr<ASSSyllabe>);

#[derive(Debug, Default, Clone)]
#[repr(transparent)]
pub struct ASSSyllabes(pub Vec<ASSSyllabePtr>);

impl PartialEq for ASSSyllabePtr {
    fn eq(&self, other: &Self) -> bool {
        *self.0.try_read().unwrap() == *other.0.try_read().unwrap()
    }
}

impl PartialEq for ASSSyllabes {
    fn eq(&self, Self(other): &Self) -> bool {
        let Self(this) = self;
        (this.len() != other.len()) && {
            this.iter()
                .zip(other.iter())
                .fold(true, |acc, (ASSSyllabePtr(this), ASSSyllabePtr(other))| {
                    acc && (*this.try_read().unwrap() == *other.try_read().unwrap())
                })
        }
    }
}

impl From<ASSSyllabe> for ASSSyllabePtr {
    fn from(value: ASSSyllabe) -> Self {
        ASSSyllabePtr(crate::ptr!(value))
    }
}

impl ASSSyllabes {
    pub fn get(&self, idx: usize) -> Option<ASSSyllabePtr> {
        self.0.get(idx).cloned()
    }

    pub fn len(&self) -> usize {
        self.0.len()
    }

    pub fn is_empty(&self) -> bool {
        self.0.is_empty()
    }

    pub fn push(&mut self, value: impl Into<ASSSyllabePtr>) {
        self.0.push(value.into())
    }
}

impl Extend<ASSSyllabePtr> for ASSSyllabes {
    fn extend<T: IntoIterator<Item = ASSSyllabePtr>>(&mut self, iter: T) {
        self.0.extend(iter)
    }
}

impl Extend<ASSSyllabe> for ASSSyllabes {
    fn extend<T: IntoIterator<Item = ASSSyllabe>>(&mut self, iter: T) {
        self.extend(iter.into_iter().map(|syllabe| ASSSyllabePtr(crate::ptr!(syllabe))))
    }
}

impl IntoIterator for ASSSyllabes {
    type Item = ASSSyllabePtr;
    type IntoIter = <Vec<ASSSyllabePtr> as IntoIterator>::IntoIter;

    fn into_iter(self) -> Self::IntoIter {
        self.0.into_iter()
    }
}
