//! ASS objects for Vivy.

#![forbid(unsafe_code)]

mod colors;
mod definitions;
mod drawing;
mod elements;
mod position;
mod reader;
mod styles;
mod types;
mod values;

#[cfg(test)]
mod tests;

pub use crate::{colors::*, drawing::*, elements::*, position::*, styles::*, types::*, values::*};
pub use reader::{ass_container_from_file, ass_container_from_str, ASSElementReaderError, ContainerFileType};

pub type Ptr<T> = std::sync::Arc<std::sync::RwLock<T>>;

#[macro_export]
macro_rules! ptr {
    ($expr: expr) => {
        std::sync::Arc::new(std::sync::RwLock::new($expr))
    };
}

/// A trait to parameterize the ASS element types. It can be a mutable pointer (Arc<RwLock>), a
/// pointer to a constant thing (Arc) or no pointer at all (just the ASS element).
pub trait ASSPtr<T>: std::ops::Deref<Target = T> {}
