// MIT License
//
// Copyright 2023 Maël MARTIN
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
// associated documentation files (the "Software"), to deal in the Software without restriction,
// including without limitation the rights to use, copy, modify, merge, publish, distribute,
// sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

grammar VVS;

chunk: topblock* mainblock EOF;

topblock: 'import' string
        : 'set' NAME '.' NAME '=' exp
        ;

mainblock:
    'main' NAME '{'
        (NAME '=' NAME '.' NAME NAME ';')*

        'write' '{' (NAME (',' NAME)* (,)?)? '}'
    '}'
;

exp: 'nil' | 'false' | 'true'
   | number
   | color
   | string
   ;

color:  '#(' ... ')';
number: INT | HEX | FLOAT | HEX_FLOAT;
string: NORMALSTRING | CHARSTRING | LONGSTRING;

// LEXER

NAME:           [a-zA-Z_][a-zA-Z_0-9]*;
NORMALSTRING:   '"' ( EscapeSequence | ~('\\'|'"') )* '"';
CHARSTRING:     '\'' ( EscapeSequence | ~('\''|'\\') )* '\'';
LONGSTRING:     '[' NESTED_STR ']';

fragment
NESTED_STR: '=' NESTED_STR '='
          | '[' .*? ']'
          ;

INT: Digit+;
HEX: '0' [xX] HexDigit+;

FLOAT: Digit+ '.' Digit* ExponentPart?
     | '.' Digit+ ExponentPart?
     | Digit+ ExponentPart
     ;

HEX_FLOAT
    : '0' [xX] HexDigit+ '.' HexDigit* HexExponentPart?
    | '0' [xX] '.' HexDigit+ HexExponentPart?
    | '0' [xX] HexDigit+ HexExponentPart
    ;

fragment
ExponentPart
    : [eE] [+-]? Digit+
    ;

fragment
HexExponentPart
    : [pP] [+-]? Digit+
    ;

fragment
EscapeSequence
    : '\\' '\r'? '\n'
    ;

fragment
Digit
    : [0-9]
    ;

fragment
HexDigit
    : [0-9a-fA-F]
    ;

fragment
SingleLineInputCharacter
    : ~[\r\n\u0085\u2028\u2029]
    ;

COMMENT
    : '--[' NESTED_STR ']' -> channel(HIDDEN)
    ;

LINE_COMMENT
    : '--' SingleLineInputCharacter* -> channel(HIDDEN)
    ;

WS
    : [ \t\u000C\r\n]+ -> skip
    ;

SHEBANG
    : '#' '!' SingleLineInputCharacter* -> channel(HIDDEN)
    ;

