use super::{tree::*, ASTStringCacheHandle};
use hashbrown::{HashMap, HashSet};
use std::{cell::RefCell, rc::Rc};

/// A VVL module of a Vivy Script
pub struct Module {
    functions: HashMap<ASTString, ASTFunction>,
    jobs: HashMap<ASTString, ASTJob>,
    consts: HashMap<ASTString, (ASTVar, ASTConst)>,
    options: HashMap<ASTString, (ASTVar, ASTConst)>,
    imports: HashSet<ASTString>,

    /// Caching for strings, identified by theyr hash and reuse the same memory location to reduce
    /// memory footprint. Use the .finish function from the hasher to get the key.
    strings: Rc<RefCell<HashMap<u64, ASTString>>>,
}

/// Here, all the getters...
impl Module {
    /// Get a handle to the string cache.
    pub fn strings(&mut self) -> ASTStringCacheHandle {
        ASTStringCacheHandle::new(self.strings.borrow_mut())
    }
}

/// Here all the setters, we try to have pure functions.
impl Module {
    /// Declare a new function.
    ///
    /// TODO: Use the entry thingy which should be better...
    pub fn declare_function(mut self, name: ASTString, function: ASTFunction) -> Self {
        if self.functions.contains_key(&name) {
            log::warn!(target: "cc", ";re-definition of function {name} at {}", function.span);
        }
        self.functions.insert(name, function);
        self
    }

    /// Declare a new job.
    ///
    /// TODO: Use the entry thingy which should be better...
    pub fn declare_job(mut self, name: ASTString, job: ASTJob) -> Self {
        if self.jobs.contains_key(&name) {
            log::warn!(target: "cc", ";re-definition of job {name} at {}", job.span);
        }
        self.jobs.insert(name, job);
        self
    }

    /// Declare a new option.
    ///
    /// TODO: Use the entry thingy which should be better...
    pub fn declare_option(self, var: ASTVar, value: ASTConst) -> Self {
        todo!()
    }

    /// Declare a new constant.
    ///
    /// TODO: Use the entry thingy which should be better...
    pub fn declare_const(self, var: ASTVar, value: ASTConst) -> Self {
        todo!()
    }

    /// Declare a new import.
    ///
    /// TODO: Use the entry thingy which should be better...
    pub fn declare_import(self, import: ASTString) -> Self {
        todo!()
    }
}

impl std::fmt::Debug for Module {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        f.debug_struct("Module")
            .field("functions", &self.functions)
            .field("jobs", &self.jobs)
            .field("consts", &self.consts)
            .field("options", &self.options)
            .field("imports", &self.imports)
            .finish()
    }
}
