use crate::{ast::*, expression, parser::vvl::*};
use vvs_utils::either;

#[allow(dead_code)]
pub(crate) fn expr<'a, 'b>(
    cache: Rc<RefCell<ASTStringCacheHandle<'b>>>,
) -> impl Fn(PSpan<'a>) -> VVParserError<'a, ASTExpr> + 'b
where
    'b: 'a,
{
    macro_rules! expr {
        // No binary expression, parse a unary expression or something else, here we have reached
        // the leaf of the tree. We are parsing some things, but we still need to handle more things:
        // - tuples
        // Stil need to write some tests!
        () => {
            |cache: Rc<RefCell<ASTStringCacheHandle<'b>>>| { move |input| -> VVParserError<'a, ASTExpr> {
                let (input, _) = multispace0(input)?;
                let (i, (rvalue, indices, method, args)) = context("expr leaf", tuple((
                    // Final leaf for the expression tree.
                    alt((
                        delimited(spaced_tag("("), expr(cache.clone()), spaced_tag(")")),
                        map(with_span(preceded(tag("$"), number)),                     |(sp, c)| expression! { sp, Const(ASTConst::Color(c)) }),
                        map(with_span(keyword("true")),                                |(sp, _)| expression! { sp, True  }),
                        map(with_span(keyword("false")),                               |(sp, _)| expression! { sp, False }),
                        map(with_span(keyword("nil")),                                 |(sp, _)| expression! { sp, Nil   }),
                        map(with_span(preceded(keyword("not"),  expr(cache.clone()))), |(sp, e)| expression! { sp, Unop(ASTUnop::LogicNot, Box::new(e)) }),
                        map(with_span(preceded(spaced_tag("#"), expr(cache.clone()))), |(sp, e)| expression! { sp, Unop(ASTUnop::Len,      Box::new(e)) }),
                        map(with_span(preceded(spaced_tag("-"), expr(cache.clone()))), |(sp, e)| expression! { sp, Unop(ASTUnop::Neg,      Box::new(e)) }),
                        map(with_span(preceded(spaced_tag("~"), expr(cache.clone()))), |(sp, e)| expression! { sp, Unop(ASTUnop::BitNot,   Box::new(e)) }),
                        map(with_span(identifier),                                     |(sp, n)| expression! { sp, Var(ASTVar(sp.into(), cache.borrow_mut().get_or_insert(n), None)) }),
                        map(with_span(string),                                         |(sp, s)| expression! { sp, Const(ASTConst::String(cache.borrow_mut().get_or_insert(s)))      }),
                        map(with_span(float),                                          |(sp, f)| expression! { sp, Const(ASTConst::Floating(f)) }),
                        map(with_span(number),                                         |(sp, i)| expression! { sp, Const(ASTConst::Integer(i))  }),
                        map(with_span(table(cache.clone())),                           |(sp, t)| expression! { sp, Table(t) }),
                        map(with_span(any_tuple(cache.clone())),                       |(sp, t)| expression! { sp, Tuple(t) }),
                    )),
                    // Do we need to index into the thing?
                    many0(alt((
                        map(delimited(spaced_tag("["), expr(cache.clone()), spaced_tag("]")), |expr| ASTField::Expr(expr.span, expr.content)),
                        map(with_span(preceded(spaced_tag("."), identifier)), |(span, field)| ASTField::Identifier(span.into(), cache.borrow_mut().get_or_insert(field))),
                    ))),
                    // Function call and method invoking stuff
                    opt(preceded(spaced_tag(":"), identifier)),
                    opt(delimited(spaced_tag("("), separated_list0(spaced_tag(","), expr(cache.clone())), spaced_tag(")"))),
                )))(input)?;
                // Build an expression from the rvalue
                let expr = either!(!indices.is_empty()
                    => expression! { input, PrefixExpr(Box::new(rvalue), indices) }
                    ;  rvalue
                );
                // Some logic to handle the call function/method thing
                let expr = match (method, args) {
                    (Some(func), Some(args)) => expression! { input, MethodInvok(Box::new(expr), cache.borrow_mut().get_or_insert(func), args) },
                    (None,       Some(args)) => expression! { input, FuncCall(Box::new(expr), args) },
                    (Some(_),    None) => todo!("a method invokation without arguments, should return the function with the first argument bounded"),
                    (None,       None) => expr,
                };
                Ok((i, expr))
            }}
        };

        // We are in a node of a binary expression. The binary operator can be a keyword or a
        // symbol list, need to handle those two cases differently.
        ( $(($str: literal, $op: ident)),+
          $(; $(($strs: literal, $ops: ident)),+)*
          $(;)?
        ) => {
            |cache: Rc<RefCell<ASTStringCacheHandle<'b>>>| { move |i| -> VVParserError<'a, ASTExpr> {
                let (input, _) = multispace0(i)?;
                let next_clozure = expr!($($(($strs, $ops)),+);*);
                let next_clozure = next_clozure(cache.clone());
                let (i, initial) = next_clozure(input)?;
                let (i, _) = multispace0(i)?;
                let (i, remainder) = many0(alt(($(
                    |i| -> VVParserError<(ASTBinop, ASTExpr)> {
                        let (i, item) = either! { $str.chars().next().unwrap().is_ascii_alphabetic()
                            => preceded(keyword($str), preceded(multispace0, next_clozure.clone()))(i)?
                            ;  preceded(tag($str),     preceded(multispace0, next_clozure.clone()))(i)?
                        };
                        Ok((i, (ASTBinop::$op, item)))
                    }
                ),+,)))(i)?;
                Ok((i, remainder.into_iter().fold(initial, |acc, (oper, expr)| expression! {
                    ASTSpan::merge(acc.span, expr.span),
                    Binop(Box::new(acc), oper, Box::new(expr))
                })))
            }}
        };
    }

    let expr = expr! [ /* 12 */ ("|",   BitOr)
                     ; /* 11 */ ("^",   BitXor)
                     ; /* 10 */ ("&",   BitAnd)
                     ; /* 09 */ ("or",  LogicOr)
                     ; /* 08 */ ("xor", LogicXor)
                     ; /* 07 */ ("and", LogicAnd)
                     ; /* 06 */ ("==",  CmpEQ), ("~=", CmpNE), ("!=", CmpNE)
                     ; /* 05 */ ("<=",  CmpLE), ("<",  CmpLT), (">=", CmpGE), (">", CmpGT)
                     ; /* 04 */ ("..",  StrCat)
                     ; /* 03 */ ("<<",  BitShiftLeft), (">>", BitShiftRight)
                     ; /* 02 */ ("+",   Add), ("-", Sub)
                     ; /* 01 */ ("*",   Mul), ("/", Div), ("%", Mod), ("mod", Mod)
                     ; /* 00 */ ("**",  Power)
    ];

    expr(cache)
}

#[test]
fn test_arithmetic() {
    use crate::{anon_expression, ast::ASTBinop::*};
    use hashbrown::HashMap;
    let strings = Rc::<RefCell<HashMap<u64, ASTString>>>::default();
    let expr = expr(Rc::new(RefCell::new(ASTStringCacheHandle::new(strings.borrow_mut()))));
    let expr = move |str: &'static str| expr(str.into());

    assert_eq!(
        expr("1+2*3").unwrap().1,
        anon_expression!(Binop(
            Box::new(anon_expression!(Const(ASTConst::Integer(1)))),
            Add,
            Box::new(anon_expression!(Binop(
                Box::new(anon_expression!(Const(ASTConst::Integer(2)))),
                Mul,
                Box::new(anon_expression!(Const(ASTConst::Integer(3))))
            )))
        ))
    );
    assert_eq!(
        expr("2*3+1").unwrap().1,
        anon_expression!(Binop(
            Box::new(anon_expression!(Binop(
                Box::new(anon_expression!(Const(ASTConst::Integer(2)))),
                Mul,
                Box::new(anon_expression!(Const(ASTConst::Integer(3))))
            ))),
            Add,
            Box::new(anon_expression!(Const(ASTConst::Integer(1)))),
        ))
    );
}

#[test]
fn test_valid_table() {
    use crate::{anon_expression, ast::*};
    use hashbrown::HashMap;
    use vvs_utils::{assert_ok, assert_some};
    let strings = Rc::<RefCell<HashMap<u64, ASTString>>>::default();
    let table = expr(Rc::new(RefCell::new(ASTStringCacheHandle::new(strings.borrow_mut()))));
    let table = move |str: &'static str| table(str.into());
    let table = assert_ok!(table(
        r#"{ toto = "tata", foo = 3.14, ["bar"] = 42, titi = {}, oupsy = nil }"#
    ));

    let table = match table.1.content {
        ASTExprVariant::Table(table) => table,
        expr => panic!("should be a table, got {expr:#?}"),
    };

    assert_eq!(
        assert_some!(table.get("toto")).content,
        anon_expression!(Const(ASTConst::String("tata".into()))).content
    );

    assert_eq!(
        assert_some!(table.get("foo")).content,
        anon_expression!(Const(ASTConst::Floating(3.14))).content
    );

    assert_eq!(
        assert_some!(table.get("bar")).content,
        anon_expression!(Const(ASTConst::Integer(42))).content
    );

    assert_eq!(
        assert_some!(table.get("titi")).content,
        anon_expression!(Table(HashMap::default())).content
    );

    assert_eq!(
        assert_some!(table.get("oupsy")).content,
        anon_expression!(Nil).content
    );
}

#[test]
fn test_redefined_key_table() {
    use crate::ast::*;
    use hashbrown::HashMap;
    use vvs_utils::assert_err;
    let strings = Rc::<RefCell<HashMap<u64, ASTString>>>::default();
    let table = expr(Rc::new(RefCell::new(ASTStringCacheHandle::new(strings.borrow_mut()))));
    let table = move |str: &'static str| table(str.into());
    assert_err!(table(r#"{ toto = "tata", toto = nil }"#));
}

#[test]
fn test_tuple_invalid() {
    use crate::ast::*;
    use hashbrown::HashMap;
    use vvs_utils::assert_err;
    let strings = Rc::<RefCell<HashMap<u64, ASTString>>>::default();
    let expr = expr(Rc::new(RefCell::new(ASTStringCacheHandle::new(strings.borrow_mut()))));
    let expr = move |str: &'static str| expr(str.into());
    match expr(r#""a", "b", "c""#) {
        Ok((i, expr)) if i.is_empty() => panic!("successfully parsed {expr:#?}"),
        _ => {}
    }
    assert_err!(expr("(1,)"));
}

#[test]
fn test_tuple() {
    use crate::{anon_expression, ast::*};
    use hashbrown::HashMap;
    use vvs_utils::assert_ok;
    let strings = Rc::<RefCell<HashMap<u64, ASTString>>>::default();
    let expr = expr(Rc::new(RefCell::new(ASTStringCacheHandle::new(strings.borrow_mut()))));
    let expr = move |str: &'static str| expr(str.into());
    assert_eq!(
        assert_ok!(expr("(1, 2, 3, 4)")).1,
        anon_expression!(Tuple(vec![
            anon_expression!(Const(ASTConst::Integer(1))),
            anon_expression!(Const(ASTConst::Integer(2))),
            anon_expression!(Const(ASTConst::Integer(3))),
            anon_expression!(Const(ASTConst::Integer(4))),
        ]))
    );
    assert_eq!(
        assert_ok!(expr("(1, 2,)")).1,
        anon_expression!(Tuple(vec![
            anon_expression!(Const(ASTConst::Integer(1))),
            anon_expression!(Const(ASTConst::Integer(2))),
        ]))
    );
}
