//! Here we define the things that will be exported to C/C++ code. We use PascalCase as it is not
//! very common and getters must be of the form `{StructName}{Get,Set}{TheField}` and methods of
//! the form `{StructName}{FunctionName}`. For free functions we use `{TheCrate}{FunctionName}`.

#![allow(non_snake_case)]

use std::ffi::c_char;

pub mod ass;
pub mod vivy;

/// Represents a string slice, the user may not modify this slice, never! Note that the string is
/// not null terminated and may contains null bytes in it, use the len attribute to get the length
/// of this string and convert it to your linking.
///
/// # Safety
/// Note that you must always put unicode inside a string slice!
#[repr(C)]
pub struct StringSlice {
    len: usize,
    str: *const c_char,
}

impl StringSlice {
    pub unsafe fn as_str(&self) -> &str {
        std::str::from_utf8_unchecked(std::slice::from_raw_parts(self.str as *const _, self.len))
    }
}

impl<S> From<S> for StringSlice
where
    S: AsRef<str>,
{
    fn from(value: S) -> Self {
        let value = value.as_ref();
        StringSlice { len: value.len(), str: value.as_ptr() as *const _ }
    }
}
