use serde::{Deserialize, Serialize};
use std::path::PathBuf;

#[derive(Debug, PartialEq, Eq)]
pub enum VivyDocumentCapabilities {
    AudioAble = (1 << 1),
    VideoAble = (1 << 2),
    AssAble = (1 << 3),
}

#[derive(Debug, Deserialize, Serialize)]
#[serde(rename_all = "PascalCase")]
pub struct VivyDocument {
    #[serde(default = "u64::default")]
    pub capabilities: u64,
    pub name: String,
}

#[derive(Debug, Deserialize, Serialize)]
#[serde(rename_all = "PascalCase")]
#[serde(tag = "DocumentVersion", content = "SubDocuments")]
pub enum VivySubDocument {
    Alpha {
        sub_ass: Option<PathBuf>,
        sub_audio: Option<PathBuf>,
        sub_video: Option<PathBuf>,
    },
}

#[test]
fn test_deserialize() {
    if let Err(err) = serde_json::from_str::<VivyDocument>(include_str!("../test/sample.vivy")) {
        panic!("{err}")
    }
}
