#[macro_export]
macro_rules! assert_eq_delta {
    ($a: expr, $b: expr, $delta: literal) => {{
        let (a, b, delta) = ($a, $b, $delta);
        let diff = (a - b);
        if !((diff <= delta && diff >= 0.0) || (-diff <= delta && diff <= 0.0)) {
            panic!(
                "assertion failed: not delta-equal: `abs({a} - {b}) > {delta}` \n\tleft: {}\n\tright: {}\n\tdelta: {}",
                stringify!($a),
                stringify!($b),
                stringify!($delta)
            );
        }
    }};
}

/// Test that something was [`Err`] and return the content of the
/// [`Result::Err`] variant. Panic on [`Ok`].
#[macro_export]
macro_rules! assert_err {
    ($x: expr) => {
        match $x {
            Ok(e) => panic!("'{}' was successfull and got '{e:?}'", stringify!($x)),
            Err(e) => e,
        }
    };
}

/// Test that something was [`Ok`] and return the content of the [`Result::Ok`]
/// variant. Panic on [`Err`].
#[macro_export]
macro_rules! assert_ok {
    ($x: expr) => {
        match $x {
            Ok(x) => x,
            Err(e) => panic!("failed '{}' with error '{e:?}'", stringify!($x)),
        }
    };
}

#[macro_export]
macro_rules! assert_some {
    ($x: expr) => {
        match $x {
            Some(x) => x,
            _ => panic!("failed '{}', expected [Some], got [None]", stringify!($x)),
        }
    };
}
