#include "TimingAxis.hh"
#include "TimingParams.hh"

using namespace Vivy;

TimingAxis::TimingAxis() noexcept
    : QGraphicsObject()
{
    pen = QPen(axisColour);
    pen.setWidth(static_cast<int>(penWidth));
}

QRectF
TimingAxis::boundingRect() const
{
    return QRectF(0, static_cast<int>(10 + penWidth),
                  static_cast<int>(getTimingParam()->audioWidth() + 2 * penWidth),
                  static_cast<int>(-2 * getTimingParam()->axisHeight() - penWidth));
}

void
TimingAxis::onParamsChanged()
{
    qsizetype nbAvailableTicks = availableTicks.size();
    qint64 minorTicksIndex     = 0;
    qint64 length              = getTimingParam()->audioLength();
    qint64 width               = getTimingParam()->audioWidth();
    if (width != 0 && length != 0) {
        for (minorTicksIndex = 0; minorTicksIndex < nbAvailableTicks; minorTicksIndex++) {
            if (width * availableTicks[minorTicksIndex] / length >= minBetweenMinor) {
                break;
            }
        }
    }

    prepareGeometryChange();
    if (minorTicksIndex < nbAvailableTicks - 1) {
        minorTicks = availableTicks[minorTicksIndex];
        majorTicks = availableTicks[minorTicksIndex + 1];
    } else {
        minorTicks = 0;
        majorTicks = availableTicks[minorTicksIndex];
    }
    getTimingParam()->setTicks(minorTicks, majorTicks);
}

void
TimingAxis::paint(QPainter *painter, const QStyleOptionGraphicsItem *, QWidget *)
{
    qint64 audioLength = getTimingParam()->audioLength();
    if (audioLength == 0)
        return;

    const int yText = static_cast<int>((-majorTicksHeight) - timeDigitsMargin);

    painter->setPen(pen);
    painter->drawLine(0, 0, static_cast<int>(audioLength), 0);

    /*
     * We may want to calculate width/audioLength
     * and audioLength/width to have just a multiplication
     * to do to get the position of each tick
     *
     * It should be faster at the cost of precision :
     * should look whether this precision loss is negligible or not
     */
    for (qint64 i = 0; i < audioLength; i += majorTicks) {
        const int pos = static_cast<int>(getTimingParam()->posFromMs(i));
        painter->drawText(QPoint(pos, yText), getTimingParam()->printMajorTicks(i));
        painter->drawLine(pos, 0, pos, static_cast<int>(-majorTicksHeight));
    }
    if (minorTicks > 0) {
        for (qint64 i = 0; i < audioLength; i += minorTicks) {
            const int pos = static_cast<int>(getTimingParam()->posFromMs(i));
            if (Q_LIKELY(fmod(i, majorTicks) < DBL_EPSILON))
                painter->drawLine(pos, 0, pos, static_cast<int>(-minorTicksHeight));
        }
    }
}
