#ifndef VIVY_TIMING_AXIS_H
#define VIVY_TIMING_AXIS_H

#ifndef __cplusplus
#error "This is a C++ header"
#endif

#include "PreCompiledHeaders.hh"
#include <cmath>

namespace Vivy
{
class TimingAxis final : public QGraphicsObject {
    Q_OBJECT

public:
    explicit TimingAxis() noexcept;
    ~TimingAxis() noexcept override = default;

    QRectF boundingRect() const override;
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) override;

private:
    static inline constexpr QColor axisColour = QColor(255, 220, 220);
    QPen pen;
    qint64 penWidth{ 1 };

    /*
     * We use ints here because
     * qPainter->drawLine() and qPainter->drawText()
     * restrict us to ints anyways
     */
    QVector<qint64> availableTicks = { 10, 100, 1000, 10000, 60000, 3600000, 86400000 };
    qint64 minBetweenMinor{ 10 };
    qint64 minorTicks;
    qint64 majorTicks;

    qint64 minorTicksHeight{ 3 };
    qint64 majorTicksHeight{ 7 };
    qint64 timeDigitsHeight{ 20 };
    qint64 timeDigitsMargin{ 3 };

public slots:
    void onParamsChanged();
};

}

#endif // VIVY_TIMING_AXIS_H
