#pragma once

#include "PreCompiledHeaders.hh"
#include "Rust/VVLib.hh"
#include "Lib/Utils.hh"
#include "Lib/Audio.hh"
#include "UI/VivyDocumentView.hh"
#include "UI/DocumentViews/AssLinesView.hh"
#include "UI/AbstractVivyDocumentNeeder.hh"

namespace Vivy
{
class TimingParams;
class TimingBar;
class TimingAxis;
class TimingCursor;
class TimingLine;

class TimingScene final : public QGraphicsScene, public AbstractVivyDocumentNeeder {
    Q_OBJECT

public:
    enum class TimingMode { Line = 0, Syl = (1 << 1), Char = (1 << 2) };
    Q_DECLARE_FLAGS(TimingModes, TimingMode)

public:
    static inline constexpr QColor startColour = QColor(127, 0, 127);
    static inline constexpr QColor endColour   = QColor(0, 127, 0);

    //explicit TimingScene(QWidget *parent = nullptr) noexcept;
    explicit TimingScene(QImage, AudioContext::StreamPtr, VivyDocumentView &,
                         QWidget * = nullptr) noexcept;

private:
    QGraphicsPixmapItem *backgroundImg{ nullptr };
    QImage img;
    TimingCursor *cursor;
    AudioContext::StreamPtr audioStream;
    VVLib::ASSLine *currentLine{ nullptr };
    TimingMode timingMode{ TimingMode::Line };

    // FIXME: add destructors
    TimingParams *params;
    TimingAxis *ax;

    QVector<TimingLine *> timingLines;

public:
    QGraphicsPixmapItem *bg() noexcept;
    void mousePressEvent(QGraphicsSceneMouseEvent *event) noexcept override;
    void mouseMoveEvent(QGraphicsSceneMouseEvent *mouseEvent) noexcept override;

    TimingAxis *getAxis();
    TimingParams *getParams();

private:
    void handleMousePressEventLine(QGraphicsSceneMouseEvent *, VVLib::ASSLine[]) noexcept;
    void handleMousePressEventSyl(QGraphicsSceneMouseEvent *, VVLib::ASSLine[]) noexcept;
    void handleMousePressEventChar(QGraphicsSceneMouseEvent *, VVLib::ASSLine[]) noexcept;

private:
    qint64 timeFromPos(qint64 x) const noexcept;
    qint64 posFromTime(qint64 t) const noexcept;

public slots:
    void updateScene(QImage, AudioContext::StreamPtr);
    void updateSelectedLines(const QItemSelection &selected, const QItemSelection &deselected);
    void rebuildScene();
};

}

//Only if combinaisons of mode is allowed, it shouldn't be here
//Q_DECLARE_OPERATORS_FOR_FLAGS(TimingScene::TimingModes)
