#pragma once

#ifndef __cplusplus
#error "This is a C++ header"
#endif

#include "PreCompiledHeaders.hh"
#include "VivyApplication.hh"
#include "Lib/Utils.hh"
#include "Lib/Log.hh"

namespace Vivy
{
class MpvContainer final : public QOpenGLWidget {
    Q_OBJECT
    VIVY_UNMOVABLE_OBJECT(MpvContainer)
    VIVY_APP_LOGGABLE_OBJECT(MpvContainer, logger)

    enum AsyncCmdType : uint64_t {
        None,
        LoadFile,
        LoadAssFile,
        ReloadAss,
        UnloadAss,
        SeekTime,
        TogglePlayback
    };

public:
    explicit MpvContainer(QWidget *parent);
    ~MpvContainer() noexcept override;

    void loadFile(const QString &) noexcept;
    void loadAssFile(const QString &) noexcept;
    void reloadAssFile() noexcept;

    void seekInFile(const chrono::seconds time) noexcept;

    // Register a callback for time change, don't use Qt's signals for that.
    // Here the function will likely be moved if necessary (I hope...).
    void registerMpvTimeCallback(std::function<void(double)>) noexcept;
    void registerMpvDurationCallback(std::function<void(double)>) noexcept;

private:
    void handleMpvEvent(const mpv_event *const) noexcept;
    void printMpvError(int) const noexcept;
    void asyncCommand(const AsyncCmdType, std::initializer_list<const char *>) noexcept;
    void unloadAssFile() noexcept;
    void handleMpvEventCommandReply(const AsyncCmdType) noexcept;
    int getAssSid() const noexcept;

    void initializeGL() override;
    void paintGL() noexcept override;

    static void onUpdate(void *) noexcept;
    static void mpvEventWakeUpCB(void *) noexcept;

    bool isPlaybackPaused{ true };
    mpv_handle *mpv{ nullptr };
    mpv_render_context *mpv_gl{ nullptr };
    qint64 sid{ -1 };
    QString previousLoadedFile{};
    std::function<void(double)> mpvTimeCallback{ nullptr };
    std::function<void(double)> mpvDurationCallback{ nullptr };

public slots:
    void mpvPlay() noexcept;
    void mpvPause() noexcept;
    void mpvTogglePlayback() noexcept;
    void maybeUpdate() noexcept;

private slots:
    void onMpvEvent() noexcept;

signals:
    void mpvEvent();
    void mpvPlaybackToggled(bool isPlaying);
};
}
