#include "PreCompiledHeaders.hh"
#include "UI/ScriptDocumentView.hh"
#include "UI/MainWindow.hh"
#include "Lib/Utils.hh"
#include "Lib/Log.hh"
#include "VivyApplication.hh"

using namespace Vivy;

ScriptDocumentView::ScriptDocumentView(std::shared_ptr<ScriptDocument> ptr, QWidget *parent)
    : AbstractDocumentView(AbstractDocumentView::Type::Script, parent)
{
    QFile textFile(ptr->getName());
    if (!textFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
        throw std::runtime_error("Failed to open script file");
    }

    VIVY_LOG_CTOR() << "Creating view for script " << VIVY_LOG_QUOTED(ptr->getName()) << " with "
                    << ptr->getUuid();
}

ScriptDocumentView::~ScriptDocumentView() { VIVY_LOG_DTOR() << "Closing a script view"; }

void
ScriptDocumentView::closeDocument() noexcept
{
}

QString
ScriptDocumentView::getDocumentTabName() const noexcept
{
    const QFileInfo fileInfo(document->getName());
    return fileInfo.completeBaseName();
}

QString
ScriptDocumentView::getDocumentTabToolTip() const noexcept
{
    return QStringLiteral("UUID: ") + document->getUuid().toString() +
           QStringLiteral("\nType: Vivy Script");
}

QIcon
ScriptDocumentView::getDocumentTabIcon() const noexcept
{
    return QIcon(VIVY_ICON_SCRIPT);
}

AbstractDocument *
ScriptDocumentView::getDocument() const noexcept
{
    return document.get();
}

void
ScriptDocumentView::openProperties() noexcept
{
}
